@@ -1,9 +1,12 @@
 from __future__ import annotations
+
+import csv
 from abc import ABC, abstractmethod
 from typing import Callable, List, Any
 from psyki.logic import Fuzzifier, Formula
 from psyki.logic.datalog.grammar import DatalogFormula, Expression, Negation, Variable, Boolean, Number, Unary, Nary, \
-    Argument, Predication
+    Argument, Predication, MofN
+from psyki.logic.datalog.grammar.adapters.antlr4 import get_formula_from_string
 
 
 class DatalogFuzzifier(Fuzzifier, ABC):
@@ -20,6 +23,7 @@ class DatalogFuzzifier(Fuzzifier, ABC):
             Boolean: self._visit_boolean,
             Number: self._visit_number,
             Unary: self._visit_unary,
+            MofN: self._visit_m_of_n,
             Nary: self._visit_nary
         }
 
@@ -58,6 +62,10 @@ class DatalogFuzzifier(Fuzzifier, ABC):
     def _visit_unary(self, formula: Formula, _) -> Any:
         pass
 
+    @abstractmethod
+    def _visit_m_of_n(self, node: MofN, local_mapping: dict[str, int] = None):
+        pass
+
     def _visit_nary(self, node: Nary, local_mapping: dict[str, int] = None):
         # Prevents side effect on the original local map.
         local_mapping_copy = self._predicates[node.name][0].copy()
@@ -78,13 +86,25 @@ class DatalogFuzzifier(Fuzzifier, ABC):
             return []
 
     def _get_predication_name(self, node: Argument):
-        if node is not None and node.arg is not None:
-            return self._get_predication_name(node.arg)
-        elif node is not None and isinstance(node.term, Predication):
-            return node.term.name
-        else:
-            return None
+        if node is not None:
+            last = node.last
+            if isinstance(last, Predication):
+                return last.name
+            elif isinstance(last, Number):
+                return str(last.value)
+            else:
+                return None
+        return None
 
     @abstractmethod
     def _clear(self):
         pass
+
+
+def load_knowledge_from_file(file: str) -> list[DatalogFormula]:
+    result = []
+    with open(str(file), mode="r", encoding="utf8") as file:
+        reader = csv.reader(file, delimiter=';')
+        for item in reader:
+            result += item
+    return [get_formula_from_string(rule) for rule in result]
