@@ -1,14 +1,20 @@
 from __future__ import annotations
 from typing import Callable
+from tensorflow.keras import Model
+from tensorflow.python.framework.ops import convert_to_tensor
 
-from psyki.logic.datalog.grammar import DatalogFormula, DefinitionClause, Clause, Expression, Variable, Boolean, Number, \
-    Unary, Negation
-from tensorflow import cast, SparseTensor, maximum, minimum, constant, reshape, reduce_max, tile
+from psyki.logic.datalog.grammar import *
+from tensorflow import cast, SparseTensor, maximum, minimum, constant, reshape, reduce_max, tile, reduce_min
 from tensorflow.python.keras.backend import to_dense
 from tensorflow.python.ops.array_ops import shape
-from psyki.logic import Formula
+from psyki.logic import Formula, get_logic_symbols_with_short_name
 from psyki.logic.datalog.fuzzifiers import ConstrainingFuzzifier
+from psyki.ski import EnrichedModel
 from psyki.utils import eta
+from psyki.utils.exceptions import SymbolicException
+
+
+_logic_symbols = get_logic_symbols_with_short_name()
 
 
 class Lukasiewicz(ConstrainingFuzzifier):
@@ -17,6 +23,7 @@ class Lukasiewicz(ConstrainingFuzzifier):
     mapping of Lukasiewicz. The resulting object is a list of continuous functions that can be used to constraint
     the predictor during its training. This is suitable for classification tasks.
     """
+    custom_objects: dict = {}
 
     def __init__(self, class_mapping: dict[str, int], feature_mapping: dict[str, int]):
         """
@@ -38,21 +45,27 @@ class Lukasiewicz(ConstrainingFuzzifier):
         self.classes: dict[str, Callable] = {}
         self._rhs: dict[str, Callable] = {}
         self._operation = {
-            '∧': lambda l, r: lambda x: eta(maximum(l(x), r(x))),
-            '∨': lambda l, r: lambda x: eta(minimum(l(x), r(x))),
-            '→': lambda l, r: lambda x: eta(l(x) - r(x)),
-            '↔': lambda l, r: lambda x: eta(abs(l(x) - r(x))),
-            '=': lambda l, r: lambda x: eta(abs(l(x) - r(x))),
-            '<': lambda l, r: lambda x: eta(constant(.5) + l(x) - r(x)),
-            '≤': lambda l, r: lambda x: eta(l(x) - r(x)),
-            '>': lambda l, r: lambda x: eta(constant(.5) - l(x) + r(x)),
-            '≥': lambda l, r: lambda x: eta(r(x) - l(x)),
+            _logic_symbols('cj'): lambda l, r: lambda x: eta(maximum(l(x), r(x))),
+            _logic_symbols('dj'): lambda l, r: lambda x: eta(minimum(l(x), r(x))),
+            _logic_symbols('e'): lambda l, r: lambda x: eta(abs(l(x) - r(x))),
+            _logic_symbols('l'): lambda l, r: lambda x: eta(constant(.5) + l(x) - r(x)),
+            _logic_symbols('le'): lambda l, r: lambda x: eta(l(x) - r(x)),
+            _logic_symbols('g'): lambda l, r: lambda x: eta(constant(.5) - l(x) + r(x)),
+            _logic_symbols('ge'): lambda l, r: lambda x: eta(r(x) - l(x)),
             'm': lambda l, r: lambda x: minimum(l(x), r(x)),
             '+': lambda l, r: lambda x: l(x) + r(x),
             '*': lambda l, r: lambda x: l(x) * r(x)
         }
+        self._aggregate_operation = {
+            _logic_symbols('cj'): lambda args: lambda x: eta(reduce_max(convert_to_tensor([arg(x) for arg in args]), axis=0)),
+            _logic_symbols('dj'): lambda args: lambda x: eta(reduce_min(convert_to_tensor([arg(x) for arg in args]), axis=0)),
+        }
         self._implication = ''
 
+    @staticmethod
+    def enriched_model(model: Model) -> EnrichedModel:
+        return EnrichedModel(model, Lukasiewicz.custom_objects)
+
     def _clear(self):
         self.classes = {}
         self._rhs = {}
@@ -97,8 +110,12 @@ class Lukasiewicz(ConstrainingFuzzifier):
                                                                                      self._visit(rhs, m)(x))))
 
     def _visit_expression(self, node: Expression, local_mapping: dict[str, int] = None) -> Callable:
-        l, r = self._visit(node.lhs, local_mapping), self._visit(node.rhs, local_mapping)
-        return self._operation.get(node.op)(l, r)
+        if len(node.nary) <= 2:
+            l, r = self._visit(node.lhs, local_mapping), self._visit(node.rhs, local_mapping)
+            return self._operation.get(node.op)(l, r)
+        else:
+            previous_layer = [self._visit(clause, local_mapping) for clause in node.nary]
+            return self._aggregate_operation.get(node.op)(previous_layer)
 
     def _visit_variable(self, node: Variable, local_mapping: dict[str, int] = None):
         if node.name in self.feature_mapping.keys():
@@ -118,4 +135,7 @@ class Lukasiewicz(ConstrainingFuzzifier):
         return self._predicates[node.name][1]({})
 
     def _visit_negation(self, node: Negation, local_mapping: dict[str, int] = None):
-        return lambda x: eta(constant(1.) - self._visit(node.predicate, local_mapping)(x))
\ No newline at end of file
+        return lambda x: eta(constant(1.) - self._visit(node.predicate, local_mapping)(x))
+
+    def _visit_m_of_n(self, node: MofN, local_mapping: dict[str, int] = None):
+        raise SymbolicException.not_supported('m of n')
