@@ -1,6 +1,6 @@
 from os.path import isdir
 from psyki.logic.datalog.grammar import DatalogFormula, Expression, DefinitionClause, Argument, Negation, Unary, Nary, \
-    Variable, Number, Predication
+    Variable, Number, Predication, MofN, ComplexArgument, optimize_datalog_formula
 from psyki.resources import PATH, create_antlr4_parser
 if not isdir(str(PATH / 'dist')):
     create_antlr4_parser(str(PATH / 'Datalog.g4'), str(PATH / 'dist'))
@@ -14,19 +14,13 @@ def get_formula_from_string(rule: str) -> DatalogFormula:
 
 
 def get_formula(ast: DatalogParser.FormulaContext) -> DatalogFormula:
-    return DatalogFormula(_get_definition_clause(ast.lhs), _get_clause(ast.rhs), ast.op.text)
+    formula: DatalogFormula = DatalogFormula(_get_definition_clause(ast.lhs), _get_clause(ast.rhs), ast.op.text)
+    optimize_datalog_formula(formula)
+    return formula
 
 
 def to_prolog_string(rule: DatalogFormula) -> str:
     # TODO: for now it works with just propositional rules without clauses except 'class'.
-    op_mapping = {
-        '≤': '=<',
-        '<': '<',
-        '≥': '>',
-        '>': '>',
-        '∧': ',',  # should not be necessary
-        '=': '=',
-    }
 
     def unfold_body(e: Expression, result=[]) -> list[str]:
         if isinstance(e.lhs, Expression):
@@ -34,7 +28,7 @@ def to_prolog_string(rule: DatalogFormula) -> str:
         if isinstance(e.rhs, Expression):
             result = result + unfold_body(e.rhs)
         if not isinstance(e.lhs, Expression) and not isinstance(e.rhs, Expression):
-            return ['\'' + op_mapping[e.op] + '\'(' + str(e.lhs) + ', ' + str(e.rhs) + ')']
+            return ['\'' + e.op + '\'(' + str(e.lhs) + ', ' + str(e.rhs) + ')']
         return result
 
     head = str(rule.lhs)
@@ -56,6 +50,13 @@ def _get_arguments(node: DatalogParser.MoreArgsContext or DatalogParser.LastTerm
         return Argument(_get_term(node.name))
 
 
+def _get_complex_arguments(node: DatalogParser.MoreComplexArgsContext or DatalogParser.LastClauseContext):
+    if isinstance(node, DatalogParser.MoreComplexArgsContext):
+        return ComplexArgument(_get_clause(node.name), _get_complex_arguments(node.args))
+    elif isinstance(node, DatalogParser.LastClauseContext):
+        return ComplexArgument(_get_clause(node.name))
+
+
 def _get_clause(node: DatalogParser.ClauseExpressionContext or DatalogParser.ClauseExpressionNoParContext
                       or DatalogParser.ClauseLiteralContext or DatalogParser.ClauseClauseContext):
     if isinstance(node, DatalogParser.ClauseExpressionContext)\
@@ -80,6 +81,8 @@ def _get_predicate(node: DatalogParser.PredicateTermContext or DatalogParser.Pre
         return _get_term(node.name)
     elif isinstance(node, DatalogParser.PredicateUnaryContext):
         return Unary(node.pred.text)
+    elif isinstance(node, DatalogParser.MofNContext):
+        return MofN(node.pred.text, node.m.text, _get_complex_arguments(node.args))
     elif isinstance(node, DatalogParser.PredicateArgsContext):
         return Nary(node.pred.text, _get_arguments(node.args))
 
