@@ -7,6 +7,7 @@ from psyki.logic.prolog.grammar import PrologFormula
 from psyki.logic import Formula, Fuzzifier
 
 
+# Not implemented yet
 class EnricherFuzzifier(Fuzzifier):
 
     def __init__(self, knowledge_base_file: str, mapping=None):
@@ -25,9 +26,7 @@ class EnricherFuzzifier(Fuzzifier):
     @staticmethod
     def _parse_file(file: str):
         knowledge_base: str = ''
-        with open(file) as f:
+        with open(file, encoding="utf8") as f:
             for row in f:
                 knowledge_base += row
         return tuprolog.theory.parsing.parse_theory(knowledge_base)
-
-
